/****************************************************************************/
/*                                                                          */
/*                   Copyright 2003 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : Checker.java                                             */
/* DESCRIPTION   : Typechecker for the Simple Expression Language           */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 28th August 2003                                         */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 28th August 2003                                         */
/****************************************************************************/

package sel.checker;

import java.util.*;
import java.io.*;
import sel.node.*;
import sel.analysis.*;
import sel.error.*;
import sel.types.*;

public class Checker extends DepthFirstAdapter {

    ErrorTable errorTable = new ErrorTable();

    public ErrorTable getErrorTable() {
	return errorTable;
    }

    public void outStart(Start node)
    {
	setOut(node, getOut(node.getPExpression()));
    }

    public void outATermExpression(ATermExpression node)
    {
        setOut(node, getOut(node.getTerm()));
    }

    public void outAExpTerm(AExpTerm node)
    {
	setOut(node, getOut(node.getExpression()));
    }

    public void outABoolLitTerm(ABoolLitTerm node)
    {
	setOut(node, new BoolType());
    }

    public void outAIntLitTerm(AIntLitTerm node)
    {
	setOut(node, new IntType());
    }

    public void outAPlusExpression(APlusExpression node)
    {
	Type leftType = (Type)getOut(node.getLeft());
	Type rightType = (Type)getOut(node.getRight());

	if (leftType != null) {
	    if (!(leftType instanceof IntType)) {
		errorTable.add(node.getPlus().getLine(),
			       new PlusLeftError(leftType.name()));
	    }
	};

	if (rightType != null) {
	    if (!(rightType instanceof IntType)) {
		errorTable.add(node.getPlus().getLine(),
			       new PlusRightError(rightType.name()));
	    }
	};

	if ((leftType instanceof IntType) 
	    && (rightType instanceof IntType)) {
	    setOut(node, new IntType());
	};
    }

    public void outAAndExpression(AAndExpression node)
    {
	Type leftType = (Type)getOut(node.getLeft());
	Type rightType = (Type)getOut(node.getRight());

	if (leftType != null) {
	    if (!(leftType instanceof BoolType)) {
		errorTable.add(node.getAnd().getLine(),
			       new AndLeftError(leftType.name()));
	    }
	};

	if (rightType != null) {
	    if (!(rightType instanceof BoolType)) {
		errorTable.add(node.getAnd().getLine(),
			       new AndRightError(rightType.name()));
	    }
	};

	if ((leftType instanceof BoolType) 
	    && (rightType instanceof BoolType)) {
	    setOut(node, new BoolType());
	};
    }

    public void outAEqExpression(AEqExpression node)
    {
	Type leftType = (Type)getOut(node.getLeft());
	Type rightType = (Type)getOut(node.getRight());

	if (leftType != null) {
	    if (!(leftType instanceof IntType)) {
		errorTable.add(node.getEq().getLine(),
			       new EqLeftError(leftType.name()));
	    }
	};

	if (rightType != null) {
	    if (!(rightType instanceof IntType)) {
		errorTable.add(node.getEq().getLine(),
			       new EqRightError(rightType.name()));
	    }
	};

	if ((leftType instanceof IntType) 
	    && (rightType instanceof IntType)) {
	    setOut(node, new BoolType());
	};
    }

    public void outACondExpression(ACondExpression node)
    {
	Type condType = (Type)getOut(node.getCond());
	Type thenType = (Type)getOut(node.getThenBranch());
	Type elseType = (Type)getOut(node.getElseBranch());

	if (condType != null) {
	    if (!(condType instanceof BoolType)) {
		errorTable.add(node.getIf().getLine(),
			       new IfCondError(condType.name()));
	    }
	};

	if ((thenType != null) && (elseType != null)) {
	    if (!(thenType.equal(elseType))) {
		errorTable.add(node.getIf().getLine(),
		    new IfMismatchError(thenType.name(),elseType.name()));
	    }
	    else {
		if (condType instanceof BoolType) {
		    setOut(node,thenType);
		}
	    }
	};

    }



}
